#include <stdio.h>
#include "opengl.h"
#include "demo.h"
#include "part.h"

void reg_whitted(void);

int win_width, win_height;
float win_aspect;
long time_msec;


int demo_init(void)
{
	int i;

	if(init_opengl() == -1) {
		return -1;
	}

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glEnable(GL_FRAMEBUFFER_SRGB);
	glEnable(GL_MULTISAMPLE);

	reg_whitted();

	for(i=0; i<num_parts; i++) {
		if(parts[i]->init() == -1) {
			fprintf(stderr, "part %s init failed\n", parts[i]->name);
			return -1;
		}
	}

	switch_part(parts[0]);
	return 0;
}

void demo_cleanup(void)
{
	int i;

	for(i=0; i<num_parts; i++) {
		parts[i]->destroy();
	}
}

void demo_display(void)
{
	long part_time;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if(!cur_part) return;

	part_time = time_msec - cur_part->start_time;
	if(part_time < cur_part->in_time) {
		float t = (float)part_time / cur_part->in_time;

		if(prev_part) {
			long prev_part_time = time_msec - prev_part->start_time;
			if(prev_part->draw_out) {
				prev_part->draw_out(prev_part_time, t);
			} else {
				prev_part->draw(prev_part_time);
			}
		}

		if(cur_part->draw_in) {
			cur_part->draw_in(part_time, t);
		} else {
			cur_part->draw(part_time);
		}
	} else {
		prev_part = 0;
		cur_part->draw(part_time);
	}
}

void demo_reshape(int x, int y)
{
	win_width = x;
	win_height = y;
	win_aspect = (float)x / (float)y;

	glViewport(0, 0, x, y);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(50.0, win_aspect, 0.5, 500.0);
}

void demo_keyboard(int key, int st)
{
	if(st && key == 27) {
		demo_quit();
		return;
	}

	if(cur_part && cur_part->keyboard) {
		cur_part->keyboard(key, st);
	}
}


void demo_mbutton(int bn, int st, int x, int y)
{
	if(cur_part && cur_part->mbutton) {
		cur_part->mbutton(bn, st, x, y);
	}
}

void demo_mmotion(int x, int y)
{
	if(cur_part && cur_part->mmotion) {
		cur_part->mmotion(x, y);
	}
}

void demo_sball_motion(int x, int y, int z)
{
}

void demo_sball_rotate(int x, int y, int z)
{
}

void demo_sball_button(int bn, int st)
{
}
